@@ -22,6 +22,20 @@ TimeoutError = socket.timeout
 
 class KombuError(Exception):
     """Common subclass for all Kombu exceptions."""
+    pass
+
+
+class SerializationError(KombuError):
+    """Failed to serialize/deserialize content."""
+
+
+class EncodeError(SerializationError):
+    """Cannot encode object."""
+    pass
+
+
+class DecodeError(SerializationError):
+    """Cannot decode object."""
 
 
 class NotBoundError(KombuError):
