@@ -18,9 +18,12 @@ except ImportError:  # pragma: no cover
     cpickle = None  # noqa
 
 from collections import namedtuple
+from contextlib import contextmanager
 
-from .exceptions import SerializerNotInstalled, ContentDisallowed
-from .five import BytesIO, text_t
+from .exceptions import (
+    ContentDisallowed, DecodeError, EncodeError, SerializerNotInstalled
+)
+from .five import BytesIO, reraise, text_t
 from .utils import entrypoints
 from .utils.encoding import str_to_bytes, bytes_t
 
@@ -44,6 +47,17 @@ pickle_protocol = int(os.environ.get('PICKLE_PROTOCOL', 2))
 codec = namedtuple('codec', ('content_type', 'content_encoding', 'encoder'))
 
 
+@contextmanager
+def _reraise_errors(wrapper,
+                    include=(Exception, ), exclude=(SerializerNotInstalled, )):
+    try:
+        yield
+    except exclude:
+        raise
+    except include as exc:
+        reraise(wrapper, wrapper(exc), sys.exc_info()[2])
+
+
 def pickle_loads(s, load=pickle_load):
     # used to support buffer objects
     return load(BytesIO(s))
@@ -133,7 +147,8 @@ class SerializerRegistry(object):
 
         # For Unicode objects, force it into a string
         if not serializer and isinstance(data, text_t):
-            payload = data.encode('utf-8')
+            with _reraise_errors(EncodeError, exclude=()):
+                payload = data.encode('utf-8')
             return 'text/plain', 'utf-8', payload
 
         if serializer:
@@ -144,7 +159,8 @@ class SerializerRegistry(object):
             content_type = self._default_content_type
             content_encoding = self._default_content_encoding
 
-        payload = encoder(data)
+        with _reraise_errors(EncodeError):
+            payload = encoder(data)
         return content_type, content_encoding, payload
     encode = dumps  # XXX compat
 
@@ -162,10 +178,12 @@ class SerializerRegistry(object):
         if data:
             decode = self._decoders.get(content_type)
             if decode:
-                return decode(data)
+                with _reraise_errors(DecodeError):
+                    return decode(data)
             if content_encoding not in SKIP_DECODE and \
                     not isinstance(data, text_t):
-                return _decode(data, content_encoding)
+                with _reraise_errors(DecodeError):
+                    return _decode(data, content_encoding)
         return data
     decode = loads  # XXX compat
 
@@ -278,7 +296,8 @@ def raw_encode(data):
     payload = data
     if isinstance(payload, text_t):
         content_encoding = 'utf-8'
-        payload = payload.encode(content_encoding)
+        with _reraise_errors(EncodeError, exclude=()):
+            payload = payload.encode(content_encoding)
     else:
         content_encoding = 'binary'
     return content_type, content_encoding, payload
